from .ipayload import IPayload
from ...errors.payload import MalformedPayloadException


class Log(IPayload):
    _valid_log_levels: list = [
        "info",
        "warning",
        "error"
    ]

    def __init__(self, level: str, message: str):
        self._level: str = level.strip()
        self._message: str = message.strip()

        if self._level not in self._valid_log_levels:
            raise MalformedPayloadException()

        if self._message.strip() == "":
            raise MalformedPayloadException()

    def to_dict(self):
        return {
            "level": self._level,
            "message": self._message
        }



import requests
from ..domain.channel import Channel
from ..domain.schemas import Result, Insight


class HTTPChannel(Channel):

    def __init__(self, url: str, api_key: str):
        self._url = url
        self._api_key = api_key

    def connect(self):
        ...

    def write(self, insight: Insight) -> Result:
        headers = {'x-api-key': self._api_key}
        response = requests.post(f"{self._url}", headers=headers, json=insight.to_dict())
        if response.status_code == 200:
            return Result(response.json())




class InvalidPayloadException(BaseException):
    message = "Invalid payload. Does not match the type of insight chosen"


class MalformedPayloadException(BaseException):
    message = "The insight payload has not been correctly parameterized"



from typing import Dict


class Result:
    def __init__(self, response: Dict):
        self._status = response.get("status", "QUEUED")
        self._tracing = response.get("tracing", "")

    def get_status(self):
        return self._status

    def get_tracing(self):
        return self._tracing



from .application.https import HTTPChannel
from .config import KRIPTOS_SERVICE_URL
from .domain.channel import Channel


class Client:
    def __init__(self, api_key: str):
        self._api_key = api_key

    def connect(self) -> Channel:
        channel = HTTPChannel(KRIPTOS_SERVICE_URL, self._api_key)
        channel.connect()
        return channel



from typing import Any

from .insigth_type import InsightType
from .payloads import Log, Event
from ..errors.payload import InvalidPayloadException


class Insight:
    def __init__(self, enterprise_id: str, insight_type: InsightType, payload: Any):
        self._enterprise_id = enterprise_id
        self._insight_type = insight_type
        self._payload = payload

        if self._insight_type == InsightType.LOG and not isinstance(payload, Log):
            raise InvalidPayloadException()

        if self._insight_type == InsightType.EVENT and not isinstance(payload, Event):
            raise InvalidPayloadException()

    def to_dict(self):
        return {
            "enterprise_id": self._enterprise_id,
            "insight_type": self._insight_type.value,
            "payload": self._payload.to_dict()
        }



from .application.aiohttps import AioHTTPChannel
from .config import KRIPTOS_SERVICE_URL
from .domain.aiochannel import AioChannel


class Client:
    def __init__(self, api_key: str):
        self._api_key = api_key

    async def connect(self) -> AioChannel:
        channel = AioHTTPChannel(KRIPTOS_SERVICE_URL, self._api_key)
        await channel.connect()
        return channel



from ..domain.channel import Channel
from ..domain.schemas import Insight, Result


class gRPCChannel(Channel):

    def __init__(self, url: str, api_key: str):
        self._url = url
        self._api_key = api_key

    def connect(self):
        ...

    def write(self, insight: Insight) -> Result:
        ...



from typing import Dict

import aiohttp

from ..domain.aiochannel import AioChannel
from ..domain.schemas import Insight, Result


class AioHTTPChannel(AioChannel):

    def __init__(self, url: str, api_key: str):
        self._url = url
        self._api_key = api_key

    async def connect(self):
        ...

    async def write(self, insight: Insight) -> Result:
        headers = {'x-api-key': self._api_key}
        async with aiohttp.ClientSession(headers=headers) as session:
            async with session.post(f"{self._url}", json=insight.to_dict()) as response:
                if response.status == 200:
                    return Result(await response.json())



from abc import ABC, abstractmethod

from .schemas import Insight
from .schemas.result import Result


class AioChannel(ABC):

    @abstractmethod
    async def connect(self): ...

    @abstractmethod
    async def write(self, insight: Insight) -> Result: ...
